USE AdventureWorksDW2008XL;
GO

--DBCC DROPCLEANBUFFERS;

SET STATISTICS TIME ON;

-- Here's a fairly standard query on the (enlarged) FactResellerSales table.
-- We want total sales by product, but only for sales made in US sales territories.
-- Dropping and ruilding the columnstore index takes too much time, so I use the
-- IGNORE_NONCLUSTERED_COLUMNSTORE_INDEX hint to mimic behaviour without CS index.
SELECT     p.EnglishProductName
		 , COUNT(rs.SalesOrderNumber)   AS  NumberOfSales
         , SUM(rs.SalesAmount)          AS  TotalSalesAmount
FROM       dbo.DimProduct               AS  p
INNER JOIN dbo.FactResellerSalesXL      AS  rs
      ON   rs.ProductKey                 =  p.ProductKey
INNER JOIN dbo.DimSalesTerritory        AS  st
      ON   st.SalesTerritoryKey          =  rs.SalesTerritoryKey
WHERE      st.SalesTerritoryCountry NOT IN('United States', 'Canada')
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName
OPTION (IGNORE_NONCLUSTERED_COLUMNSTORE_INDEX);

--DBCC DROPCLEANBUFFERS;

-- Here is the exact same query, now with CS index.
SELECT     p.EnglishProductName
		 , COUNT(rs.SalesOrderNumber)   AS  NumberOfSales
         , SUM(rs.SalesAmount)          AS  TotalSalesAmount
FROM       dbo.DimProduct               AS  p
INNER JOIN dbo.FactResellerSalesXL      AS  rs
      ON   rs.ProductKey                 =  p.ProductKey
INNER JOIN dbo.DimSalesTerritory        AS  st
      ON   st.SalesTerritoryKey          =  rs.SalesTerritoryKey
WHERE      st.SalesTerritoryCountry NOT IN('United States', 'Canada')
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName;

-- Execution times with "cold" cache: 131259 ms vs 1729 ms = 76 x faster
-- Execution times with "hot" cache:   11353 ms vs  273 ms = 42 x faster
GO




-- But are these comparisons even fair?
-- What if we add a "regular" (rowstore) covering index?

-- Building the index took over 29 minutes on my laptop
-- (mostly due to the sort operator spilling to tempdb),
-- so I can't demonstrate it during the session;
-- you'll have to take my word for the performance claims below.
/*
CREATE NONCLUSTERED INDEX IX_STK
ON [dbo].[FactResellerSalesXL] ([SalesTerritoryKey])
INCLUDE ([ProductKey],[SalesAmount]);
*/

-- After creating the nonclustered index above:
-- Execution times with "cold" cache: 50774 ms vs 1746 ms = 29 x faster
-- Execution times with "hot" cache:   5323 ms vs  296 ms = 18 x faster

-- But to be honest - can you you really afford to have
-- a covering index for every single query?
-- A single columnstore index WILL help all queries,
-- because all columns are (ideally) included in it.
-- So realistically, most queries will NOT have a covering index.
-- That will be the assumption for the rest of the presentation.
GO
