USE AdventureWorksDW2008XL;
GO

SET STATISTICS TIME ON;

--DBCC DROPCLEANBUFFERS;

-- The workaround for the limitation on outer joins is based on
-- the fact that the aggregation is only relevant for products
-- that were actually sold.
-- So we can use a 2 step approach:
-- First calculate aggregates using inner join (processing most rows in batch mode).
-- Then use outer join to add other products (row mode, but for few rows only).

-- Note the derived table to get distinct product names;
-- this is required here because product names are not unique in DimProducts.
WITH AggregatedPerProduct
AS (SELECT     p.EnglishProductName
             , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
             , SUM(rs.SalesAmount)        AS  TotalSalesAmount
    FROM       dbo.DimProduct             AS  p
    INNER JOIN dbo.FactResellerSalesXL    AS  rs
          ON   rs.ProductKey               =  p.ProductKey
          AND  rs.SalesTerritoryKey       >=  7
    GROUP BY   p.EnglishProductName)
SELECT     p.EnglishProductName
		 , COALESCE(a.NumberOfSales,0)    AS  NumberOfSales
		 , a.TotalSalesAmount             AS  TotalSalesAmount
FROM      (SELECT DISTINCT EnglishProductName
           FROM dbo.DimProduct)           AS  p
LEFT  JOIN AggregatedPerProduct           AS  a
      ON   a.EnglishProductName            =  p.EnglishProductName
ORDER BY   p.EnglishProductName;

-- Execution times with "cold" cache: 128392 ms vs 1859 ms = 69 x faster
-- Execution times with "hot" cache:   28636 ms vs  364 ms = 79 x faster
-- (Note: Comparisons with original rowstore based outer join query).



-- Here is an alternative version;
-- in this case, the CTE pre-aggregates on ProductKey (unique for product);
-- the main query then joins and aggregates on EnglishProductName
-- There is no significant performance difference with the previous version.
WITH AggregatedPerProductKey
AS (SELECT     rs.ProductKey
             , COALESCE(COUNT(rs.SalesOrderNumber), 0)
			                              AS  NumberOfSales
             , SUM(rs.SalesAmount)        AS  TotalSalesAmount
    FROM       dbo.FactResellerSalesXL    AS  rs
    WHERE      rs.SalesTerritoryKey       >=  7
    GROUP BY   rs.ProductKey)
SELECT     p.EnglishProductName
		 , SUM(a.NumberOfSales)           AS  NumberOfSales
		 , SUM(a.TotalSalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct                 AS  p
LEFT  JOIN AggregatedPerProductKey        AS  a
      ON   a.ProductKey                    =  p.ProductKey
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName;

-- Yet another rewrite, based on the observation that with a different GROUP BY,
-- the optimizer sometimes does produce a plan that uses batch mode.
-- This is a hit and miss affair, as I don't know WHY some columns
-- in the GROUP BY will result in a batch mode plan.
-- There is no significant performance difference with the previous versions.
-- The basic pattern: if you want GROUP BY X, and GROUP BY Y runs in batch mode,
-- then use a CTE with GROUP BY X, Y, then do GROUP BY X in the outer query.
-- Using GROUP BY Y, X in the CTE should be equivalent, but is (slightly) slower.
WITH ExtraGrouping
AS (SELECT     p.EnglishProductName
             , p.ProductKey
             , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
             , SUM(rs.SalesAmount)        AS  TotalSalesAmount
    FROM       dbo.DimProduct             AS  p
    LEFT  JOIN dbo.FactResellerSalesXL    AS  rs
          ON   rs.ProductKey               =  p.ProductKey
          AND  rs.SalesTerritoryKey       >=  7
    GROUP BY   p.EnglishProductName
             , p.ProductKey)
SELECT     e.EnglishProductName
		 , SUM(e.NumberOfSales)           AS  NumberOfSales
		 , SUM(e.TotalSalesAmount)        AS  TotalSalesAmount
FROM       ExtraGrouping                  AS  e
GROUP BY   e.EnglishProductName
ORDER BY   e.EnglishProductName;
