USE AdventureWorksDW2008Big;
GO

SET STATISTICS TIME ON;
GO

-- Prepare the staging table to receive data for the next month:
-- Drop the columnstore index; change the CHECK constraint.
DROP INDEX dbo.FactResellerSalesSTG.CSI_FactResellerSalesSTG;

ALTER TABLE dbo.FactResellerSalesSTG
DROP CONSTRAINT CK_Correct_Partition;

ALTER TABLE dbo.FactResellerSalesSTG
WITH CHECK
ADD CONSTRAINT CK_Correct_Partition
    CHECK (OrderDateKey >= 20040801 AND OrderDateKey < 20040901);
GO

-- Insert some rows in the staging table, for realism.
INSERT INTO dbo.FactResellerSalesSTG
SELECT  ProductKey
      , 20040825        -- Diffetent OrderDateKey, for recent dates
      , DueDateKey
      , ShipDateKey
      , ResellerKey
      , EmployeeKey
      , PromotionKey
      , CurrencyKey
      , SalesTerritoryKey
      , SalesOrderNumber
      , SalesOrderLineNumber
      , RevisionNumber
      , OrderQuantity
      , UnitPrice
      , ExtendedAmount
      , UnitPriceDiscountPct
      , DiscountAmount
      , ProductStandardCost
      , TotalProductCost
      , SalesAmount
      , TaxAmt
      , Freight
      , CarrierTrackingNumber
      , CustomerPONumber
FROM    dbo.FactResellerSalesPart
ORDER BY OrderDateKey
         OFFSET 0 ROWS
		 FETCH FIRST 500 ROWS ONLY;

SELECT *
FROM   dbo.FactResellerSalesSTG;
GO


-- With most data in the columnstore indexed table,
-- and recent data in the staging table,
-- creating a view to combined them is normally a good option.
CREATE VIEW FactResellerSalesALL
AS
  SELECT  ProductKey
        , OrderDateKey
        , DueDateKey
        , ShipDateKey
        , ResellerKey
        , EmployeeKey
        , PromotionKey
        , CurrencyKey
        , SalesTerritoryKey
        , SalesOrderNumber
        , SalesOrderLineNumber
        , RevisionNumber
        , OrderQuantity
        , UnitPrice
        , ExtendedAmount
        , UnitPriceDiscountPct
        , DiscountAmount
        , ProductStandardCost
        , TotalProductCost
        , SalesAmount
        , TaxAmt
        , Freight
        , CarrierTrackingNumber
        , CustomerPONumber
  FROM   dbo.FactResellerSalesPart
  UNION ALL
  SELECT  ProductKey
        , OrderDateKey
        , DueDateKey
        , ShipDateKey
        , ResellerKey
        , EmployeeKey
        , PromotionKey
        , CurrencyKey
        , SalesTerritoryKey
        , SalesOrderNumber
        , SalesOrderLineNumber
        , RevisionNumber
        , OrderQuantity
        , UnitPrice
        , ExtendedAmount
        , UnitPriceDiscountPct
        , DiscountAmount
        , ProductStandardCost
        , TotalProductCost
        , SalesAmount
        , TaxAmt
        , Freight
        , CarrierTrackingNumber
        , CustomerPONumber
  FROM   dbo.FactResellerSalesSTG;
GO

-- This query is pretty fast ...
-- (Note that Canada is now included as well,
--  because SalesTerritoryID >= 7 happens not to be in the staging table,
--  and I wanted some rows to actually come from the staging table).
SELECT     p.EnglishProductName
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesALL   AS  rs
      ON   rs.ProductKey               =  p.ProductKey
WHERE      rs.SalesTerritoryKey       >=  6
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName;
GO

-- ... but this one is not.
-- It turns out this one does a lot of work in row mode!
SELECT     p.EnglishProductName
         , r.ResellerName
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesALL   AS  rs
      ON   rs.ProductKey               =  p.ProductKey
INNER JOIN dbo.DimReseller            AS  r
      ON   r.ResellerKey               =  rs.ResellerKey
WHERE      rs.SalesTerritoryKey       >=  6
GROUP BY   p.EnglishProductName
         , r.ResellerName
ORDER BY   p.EnglishProductName
         , r.ResellerName;
GO

-- The workaround is not to use the view,
-- but use two queries (one for the CS table,
-- the other for the staging table) and combined them later..
WITH AggrCS AS
 (SELECT     p.EnglishProductName
           , r.ResellerName
           , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
           , SUM(rs.SalesAmount)        AS  TotalSalesAmount
  FROM       dbo.DimProduct             AS  p
  INNER JOIN dbo.FactResellerSalesPart  AS  rs
        ON   rs.ProductKey               =  p.ProductKey
  INNER JOIN dbo.DimReseller            AS  r
        ON   r.ResellerKey               =  rs.ResellerKey
  WHERE      rs.SalesTerritoryKey       >=  6
  GROUP BY   p.EnglishProductName
           , r.ResellerName)
,AggrSTG AS
 (SELECT     p.EnglishProductName
           , r.ResellerName
           , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
           , SUM(rs.SalesAmount)        AS  TotalSalesAmount
  FROM       dbo.DimProduct             AS  p
  INNER JOIN dbo.FactResellerSalesSTG   AS  rs
        ON   rs.ProductKey               =  p.ProductKey
  INNER JOIN dbo.DimReseller            AS  r
        ON   r.ResellerKey               =  rs.ResellerKey
  WHERE      rs.SalesTerritoryKey       >=  6
  GROUP BY   p.EnglishProductName
           , r.ResellerName)
,AggrALL AS
 (SELECT * FROM AggrCS
  UNION ALL
  SELECT * FROM AggrSTG)
SELECT    EnglishProductName
        , ResellerName
        , SUM(NumberOfSales)            AS  NumberOfSales
		, SUM(TotalSalesAmount)         AS  TotalSalesAmount
FROM     AggrALL
GROUP BY EnglishProductName
       , ResellerName
ORDER BY EnglishProductName
       , ResellerName;;
GO
