USE AdventureWorksDW2008XL;
GO

SET STATISTICS TIME ON;

-- Let's check the same old query again.
-- But this time, the sales territories of interest are in a special table
IF EXISTS
 (SELECT *
  FROM   sys.objects
  WHERE  object_id = OBJECT_ID(N'dbo.MyTerritories')
  AND    type = N'U')
BEGIN;
  DROP TABLE dbo.MyTerritories;
END;
GO

CREATE TABLE dbo.MyTerritories
   (SalesTerritoryKey int NOT NULL PRIMARY KEY);
GO

INSERT INTO dbo.MyTerritories (SalesTerritoryKey)
VALUES (1), (3), (5), (8), (13);
GO


-- Using IN does give us a batch mode plan,
-- but the predicate is not pushed down.
-- (Check actual rows coming from Columnstore Index Scan iterator)
SELECT     p.EnglishProductName
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
WHERE      rs.SalesTerritoryKey       IN (SELECT SalesTerritoryKey
                                          FROM   dbo.MyTerritories)
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName;

-- And if the query is just a bit more complex
-- (one extra join; one extra group by level),
-- the same IN condition even causes a fallback to row mode!
SELECT     p.EnglishProductName
         , pr.DiscountPct
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
INNER JOIN dbo.DimPromotion           AS  pr
      ON   pr.PromotionKey             =  rs.PromotionKey
WHERE      rs.SalesTerritoryKey       IN (SELECT SalesTerritoryKey
                                          FROM   dbo.MyTerritories)
GROUP BY   p.EnglishProductName
         , pr.DiscountPct
ORDER BY   p.EnglishProductName
         , pr.DiscountPct;

-- Using EXISTS has the same problems: ...
-- ... predicate not pushed down for simple query, ...
SELECT     p.EnglishProductName
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
WHERE EXISTS
 (SELECT   *
  FROM     dbo.MyTerritories          AS  t
  WHERE    t.SalesTerritoryKey         =  rs.SalesTerritoryKey)
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName;

-- ... and fallback to row mode for more complex queries.
SELECT     p.EnglishProductName
         , pr.DiscountPct
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
INNER JOIN dbo.DimPromotion           AS  pr
      ON   pr.PromotionKey             =  rs.PromotionKey
WHERE EXISTS
 (SELECT   *
  FROM     dbo.MyTerritories          AS  t
  WHERE    t.SalesTerritoryKey         =  rs.SalesTerritoryKey)
GROUP BY   p.EnglishProductName
         , pr.DiscountPct
ORDER BY   p.EnglishProductName
         , pr.DiscountPct;


-- But using INNER JOIN does improve performance;
-- this time the predicate does get pushed down.
SELECT     p.EnglishProductName
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
INNER JOIN dbo.MyTerritories          AS  t
      ON   t.SalesTerritoryKey         =  rs.SalesTerritoryKey
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName;

-- An INNER JOIN also speeds up the more complex query (a lot!)
SELECT     p.EnglishProductName
         , pr.DiscountPct
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
INNER JOIN dbo.DimPromotion           AS  pr
      ON   pr.PromotionKey             =  rs.PromotionKey
INNER JOIN dbo.MyTerritories          AS  t
      ON   t.SalesTerritoryKey         =  rs.SalesTerritoryKey
GROUP BY   p.EnglishProductName
         , pr.DiscountPct
ORDER BY   p.EnglishProductName
         , pr.DiscountPct;



-- Note: If duplicates are allowed in dbo.MyTerritories,
-- you need to join to a derived table with DISTINCT;
-- otherwise, you will get duplicate rows in the result of the join;
-- the GROUP BY will hide them, but they will count double!
DROP TABLE dbo.MyTerritories;
GO

CREATE TABLE dbo.MyTerritories
   (SalesTerritoryKey int NOT NULL);
GO

INSERT INTO dbo.MyTerritories (SalesTerritoryKey)
VALUES (1), (3), (3), (3), (5), (5), (5), (5), (5), (8), (13);
GO

WITH DistinctTerritories
AS (SELECT DISTINCT SalesTerritoryKey
    FROM   dbo.MyTerritories)
SELECT     p.EnglishProductName
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
INNER JOIN DistinctTerritories        AS  t
      ON   t.SalesTerritoryKey         =  rs.SalesTerritoryKey
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName;

-- Without the DISTINCT, the results are wrong - and slower!
SELECT     p.EnglishProductName
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
INNER JOIN dbo.MyTerritories          AS  t
      ON   t.SalesTerritoryKey         =  rs.SalesTerritoryKey
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName;
