USE AdventureWorksDW2008XL;
GO

SET STATISTICS TIME ON;

-- So how about the reverse - we EXCLUDE sales territories in a special table
IF EXISTS
 (SELECT *
  FROM   sys.objects
  WHERE  object_id = OBJECT_ID(N'dbo.MyTerritories')
  AND    type = N'U')
BEGIN;
  DROP TABLE dbo.MyTerritories;
END;
GO

CREATE TABLE dbo.MyTerritories
   (SalesTerritoryKey int NOT NULL PRIMARY KEY);
GO

INSERT INTO dbo.MyTerritories (SalesTerritoryKey)
VALUES (1), (3), (5), (8), (13);
GO


-- Both NOT IN and NOT EXISTS result in a plan that uses row mode.
SELECT     p.EnglishProductName
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
WHERE      rs.SalesTerritoryKey   NOT IN (SELECT SalesTerritoryKey
                                          FROM   dbo.MyTerritories)
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName;

SELECT     p.EnglishProductName
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
WHERE NOT EXISTS
 (SELECT   *
  FROM     dbo.MyTerritories          AS  t
  WHERE    t.SalesTerritoryKey         =  rs.SalesTerritoryKey)
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName;

-- One possible workaround: Pre-aggregate all data,
-- then exclude the listed territories
WITH AggregatedPerProduct
AS (SELECT     p.EnglishProductName
             , rs.SalesTerritoryKey
             , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
             , SUM(rs.SalesAmount)        AS  TotalSalesAmount
    FROM       dbo.DimProduct             AS  p
    INNER JOIN dbo.FactResellerSalesXL    AS  rs
          ON   rs.ProductKey               =  p.ProductKey
    GROUP BY   p.EnglishProductName
	         , rs.SalesTerritoryKey)
SELECT     a.EnglishProductName
		 , SUM(a.NumberOfSales)           AS  NumberOfSales
		 , SUM(a.TotalSalesAmount)        AS  TotalSalesAmount
FROM       AggregatedPerProduct           AS  a
WHERE      a.SalesTerritoryKey        NOT IN (SELECT SalesTerritoryKey
                                              FROM   dbo.MyTerritories)
GROUP BY   a.EnglishProductName
ORDER BY   a.EnglishProductName;

-- And another possible workaround:
-- Change the NOT IN to an IN by finding all "other" territories.
-- This results in two scans, but both in batch mode,
-- and one for only the SalesTerritoryKey column;
-- Surprisingly, in spite of the double full scann of the CS index,
-- this version is consistently FASTER than the one above!
-- (Even though the percentages in the execution plan -based on ESTIMATES!- say otherwise)
WITH AllTerritories
AS (SELECT DISTINCT SalesTerritoryKey
    FROM   dbo.FactResellerSalesXL)
,NeededTerritories
AS (SELECT SalesTerritoryKey
    FROM   AllTerritories
	WHERE  SalesTerritoryKey      NOT IN (SELECT SalesTerritoryKey
	                                      FROM   dbo.MyTerritories))
SELECT     p.EnglishProductName
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
WHERE      rs.SalesTerritoryKey       IN (SELECT SalesTerritoryKey
                                          FROM   NeededTerritories)
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName;

-- And of course, the next step is to convert the IN to an INNER JOIN,
-- so that the predicate is pushed down again;
-- this makes the query go even faster.
WITH AllTerritories
AS (SELECT DISTINCT SalesTerritoryKey
    FROM   dbo.FactResellerSalesXL)
,NeededTerritories
AS (SELECT SalesTerritoryKey
    FROM   AllTerritories
	WHERE  SalesTerritoryKey      NOT IN (SELECT SalesTerritoryKey
	                                      FROM   dbo.MyTerritories))
SELECT     p.EnglishProductName
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
INNER JOIN NeededTerritories          AS  t
      ON   t.SalesTerritoryKey         =  rs.SalesTerritoryKey
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName;
