USE AdventureWorksDW2008XL;
GO

SET STATISTICS TIME ON;

-- Now we want a report on sales made in Germany
-- or where a discount of up to 0.1 percent was given.
-- We get batch mode, but no predicate is pushed down.
SELECT     p.EnglishProductName
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
INNER JOIN dbo.DimSalesTerritory      AS  t
      ON   t.SalesTerritoryKey         =  rs.SalesTerritoryKey
INNER JOIN dbo.DimPromotion           AS  pr
      ON   pr.PromotionKey             =  rs.PromotionKey
WHERE      t.SalesTerritoryCountry     = 'Germany'
OR         pr.DiscountPct             <=  0.1
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName;


-- The somewhat obvious first attempt to work around the issue:
-- get sales from Germany, combine with sales with <= 0.1 discount.
-- However, this is incorrect: sales in Germany with a <= 0.1 percent discount are counted twice!
SELECT       EnglishProductName
            ,SUM(NumberOfSales)           AS  NumberOfSales
           , SUM(TotalSalesAmount)        AS  TotalSalesAmount
FROM
 (SELECT     p.EnglishProductName
           , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
           , SUM(rs.SalesAmount)        AS  TotalSalesAmount
  FROM       dbo.DimProduct             AS  p
  INNER JOIN dbo.FactResellerSalesXL    AS  rs
        ON   rs.ProductKey               =  p.ProductKey
  INNER JOIN dbo.DimSalesTerritory      AS  t
        ON   t.SalesTerritoryKey         =  rs.SalesTerritoryKey
  WHERE      t.SalesTerritoryCountry     = 'Germany'
  GROUP BY   p.EnglishProductName
  UNION ALL
  SELECT     p.EnglishProductName
           , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
           , SUM(rs.SalesAmount)        AS  TotalSalesAmount
  FROM       dbo.DimProduct             AS  p
  INNER JOIN dbo.FactResellerSalesXL    AS  rs
        ON   rs.ProductKey               =  p.ProductKey
  INNER JOIN dbo.DimPromotion           AS  pr
        ON   pr.PromotionKey             =  rs.PromotionKey
  WHERE      pr.DiscountPct             <=  0.1
  GROUP BY   p.EnglishProductName)      AS  der
GROUP BY     EnglishProductName
ORDER BY     EnglishProductName;

-- Using UNION instead of UNION ALL does not fix the errors!
-- To fix the errors, we would need to remove duplicates
-- before the totals are aggregated; not after that.
SELECT       EnglishProductName
            ,SUM(NumberOfSales)           AS  NumberOfSales
           , SUM(TotalSalesAmount)        AS  TotalSalesAmount
FROM
 (SELECT     p.EnglishProductName
           , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
           , SUM(rs.SalesAmount)        AS  TotalSalesAmount
  FROM       dbo.DimProduct             AS  p
  INNER JOIN dbo.FactResellerSalesXL    AS  rs
        ON   rs.ProductKey               =  p.ProductKey
  INNER JOIN dbo.DimSalesTerritory      AS  t
        ON   t.SalesTerritoryKey         =  rs.SalesTerritoryKey
  WHERE      t.SalesTerritoryCountry     = 'Germany'
  GROUP BY   p.EnglishProductName
  UNION
  SELECT     p.EnglishProductName
           , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
           , SUM(rs.SalesAmount)        AS  TotalSalesAmount
  FROM       dbo.DimProduct             AS  p
  INNER JOIN dbo.FactResellerSalesXL    AS  rs
        ON   rs.ProductKey               =  p.ProductKey
  INNER JOIN dbo.DimPromotion           AS  pr
        ON   pr.PromotionKey             =  rs.PromotionKey
  WHERE      pr.DiscountPct             <=  0.1
  GROUP BY   p.EnglishProductName)      AS  der
GROUP BY     EnglishProductName
ORDER BY     EnglishProductName;

-- Using UNION before aggregating does fix the errors - but performs terrible!
-- That's because the UNION between two huge collections of rows,
-- and the aggregations after that, are now all in row mode!
SELECT       EnglishProductName
           , COUNT(SalesOrderNumber)    AS  NumberOfSales
           , SUM(SalesAmount)           AS  TotalSalesAmount
FROM
 (SELECT     rs.SalesOrderNumber
           , rs.SalesOrderLineNumber
		   , rs.SalesAmount
		   , p.EnglishProductName
  FROM       dbo.DimProduct             AS  p
  INNER JOIN dbo.FactResellerSalesXL    AS  rs
        ON   rs.ProductKey               =  p.ProductKey
  INNER JOIN dbo.DimSalesTerritory      AS  t
        ON   t.SalesTerritoryKey         =  rs.SalesTerritoryKey
  WHERE      t.SalesTerritoryCountry     = 'Germany'
  UNION
  SELECT     rs.SalesOrderNumber
           , rs.SalesOrderLineNumber
		   , rs.SalesAmount
		   , p.EnglishProductName
  FROM       dbo.DimProduct             AS  p
  INNER JOIN dbo.FactResellerSalesXL    AS  rs
        ON   rs.ProductKey               =  p.ProductKey
  INNER JOIN dbo.DimPromotion           AS  pr
        ON   pr.PromotionKey             =  rs.PromotionKey
  WHERE      pr.DiscountPct             <=  0.1)
                                        AS  der
GROUP BY     EnglishProductName
ORDER BY     EnglishProductName;

-- The first attempt was actually not bad at all!
-- The "correct" fix is to add a filter on the second query
-- to remove all German sales from the sales with a <= 0.1 discount.
-- Now no duplicates can remain, and UNION ALL can safely be used.
-- NOTE: When using this technique, beware of nullable columns!
-- Remember, a country of NULL is neither = 'Germany', nor <> 'Germany' !!!
SELECT       EnglishProductName
            ,SUM(NumberOfSales)           AS  NumberOfSales
           , SUM(TotalSalesAmount)        AS  TotalSalesAmount
FROM
 (SELECT     p.EnglishProductName
           , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
           , SUM(rs.SalesAmount)        AS  TotalSalesAmount
  FROM       dbo.DimProduct             AS  p
  INNER JOIN dbo.FactResellerSalesXL    AS  rs
        ON   rs.ProductKey               =  p.ProductKey
  INNER JOIN dbo.DimSalesTerritory      AS  t
        ON   t.SalesTerritoryKey         =  rs.SalesTerritoryKey
  WHERE      t.SalesTerritoryCountry     = 'Germany'
  GROUP BY   p.EnglishProductName
  UNION ALL
  SELECT     p.EnglishProductName
           , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
           , SUM(rs.SalesAmount)        AS  TotalSalesAmount
  FROM       dbo.DimProduct             AS  p
  INNER JOIN dbo.FactResellerSalesXL    AS  rs
        ON   rs.ProductKey               =  p.ProductKey
  INNER JOIN dbo.DimSalesTerritory      AS  t
        ON   t.SalesTerritoryKey         =  rs.SalesTerritoryKey
  INNER JOIN dbo.DimPromotion           AS  pr
        ON   pr.PromotionKey             =  rs.PromotionKey
  WHERE      pr.DiscountPct             <=  0.1
  AND        t.SalesTerritoryCountry    <> 'Germany'
  GROUP BY   p.EnglishProductName)      AS  der
GROUP BY     EnglishProductName
ORDER BY     EnglishProductName;
