USE AdventureWorksDW2008XL;
GO

SET STATISTICS TIME ON;

-- So now we need to find how many employees,
-- and how many resellers were involved in non-North America sales.
-- This query is incredibly slow - running for almost 3.5 minutes!
-- Check the execution plan to see why.
-- Two DISTINCT aggregates cause the query optimizer to do two bad things to our plan:
-- 1. Spool intermediate results to tempdb for later reuse - slower than recomputing with CS and batch mode
-- 2. Put most of the plan in row mode (because spools don't work in batch mode).
SELECT     p.EnglishProductName
		 , COUNT(DISTINCT rs.EmployeeKey) 
		                              AS  NumEmployees
		 , COUNT(DISTINCT rs.ResellerKey)
		                              AS  NumResellers
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
WHERE      rs.SalesTerritoryKey       >=  7
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName;

-- The workaround, as usual, is to break up the query in smaller snippets;
-- then combined the subresults to one final result.
WITH DistinctEmployees AS
 (SELECT     p.EnglishProductName
           , COUNT(DISTINCT rs.EmployeeKey) 
		                                AS  NumEmployees
  FROM       dbo.DimProduct             AS  p
  INNER JOIN dbo.FactResellerSalesXL    AS  rs
        ON   rs.ProductKey               =  p.ProductKey
  WHERE      rs.SalesTerritoryKey       >=  7
  GROUP BY   p.EnglishProductName),
DistinctResellers AS
 (SELECT     p.EnglishProductName
           , COUNT(DISTINCT rs.ResellerKey)
		                                AS  NumResellers
  FROM       dbo.DimProduct             AS  p
  INNER JOIN dbo.FactResellerSalesXL    AS  rs
        ON   rs.ProductKey               =  p.ProductKey
  WHERE      rs.SalesTerritoryKey       >=  7
  GROUP BY   p.EnglishProductName)
SELECT       e.EnglishProductName,
             e.NumEmployees,
			 r.NumResellers
FROM         DistinctEmployees          AS  e
INNER JOIN   DistinctResellers          AS  r
      ON     r.EnglishProductName        =  e.EnglishProductName
ORDER BY     e.EnglishProductName;


-- We also get bad performance when combining a DISTINCT aggregate with a "normal" aggregate,
-- but in this case the cause is different.
-- The plan switches to row mode long before the number of rows is reduced,
-- which causes the query to be slow.
-- Note that there are no spools in this time, so it's not quite as bad as the one before.
SELECT     p.EnglishProductName
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , COUNT(DISTINCT rs.ResellerKey)
                                      AS  NumResellers
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
WHERE      rs.SalesTerritoryKey       >=  7
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName;

-- The workaround is the same.
WITH NumSales AS
 (SELECT     p.EnglishProductName
		   , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
  FROM       dbo.DimProduct             AS  p
  INNER JOIN dbo.FactResellerSalesXL    AS  rs
        ON   rs.ProductKey               =  p.ProductKey
  WHERE      rs.SalesTerritoryKey       >=  7
  GROUP BY   p.EnglishProductName),
DistinctResellers AS
 (SELECT     p.EnglishProductName
           , COUNT(DISTINCT rs.ResellerKey)
		                                AS  NumResellers
  FROM       dbo.DimProduct             AS  p
  INNER JOIN dbo.FactResellerSalesXL    AS  rs
        ON   rs.ProductKey               =  p.ProductKey
  WHERE      rs.SalesTerritoryKey       >=  7
  GROUP BY   p.EnglishProductName)
SELECT       s.EnglishProductName,
             s.NumberOfSales,
			 r.NumResellers
FROM         NumSales                   AS  s
INNER JOIN   DistinctResellers          AS  r
      ON     r.EnglishProductName        =  s.EnglishProductName
ORDER BY     s.EnglishProductName;
